package background;

import java.util.ArrayList;

import background.systemVariables.SimpleVariable;

public class AddressSpace {

	protected ArrayList<Point> space = new ArrayList<Point>();
	protected VariableManager vm;
	
	public AddressSpace(VariableManager v) {
		vm = v;
	}
	
	public void reset() {
		space.clear();
	}
	

	public String getVariable(int address) {
		int sum = 0;
		int i;
		
		for (i=0; i < space.size() && sum < address; i++) {
			sum += space.get(i).size * space.get(i).size2;
		}
		
		if ((i >= space.size() && sum < address) || address < 0) {
			vm.errorHandler.errorHappened("PointerRange");
			return "inf";
		}
		
		if (sum > address || space.get(i).size != 1) {
			if (sum > address) i--;
			if (i < 0) sum = 0;
			else if (address != sum) sum = sum - (space.get(i).size * space.get(i).size2);
			
			int diff = address - sum;
			int a = (diff / space.get(i).size) + space.get(i).posun2;
			int b = (diff % space.get(i).size) + space.get(i).posun;
			
			if (b > space.get(i).size) return "";
			
			String s = space.get(i).name + "[" + b + "]";
			if (space.get(i).size2 != 1) s = s + "[" + a + "]";
			
			return s;
			
		} else if (i < space.size()) return space.get(i).name;
		else return "";
	}

	public SimpleVariable getPosition(String name) {
		if (name == null) {
			AddressCell cell = new AddressCell();
			cell.cell = 0;
			cell.initialized = true;
			return cell;
		}
		if (name.equals("inf")) {
			AddressCell cell = new AddressCell();
			cell.cell = 1000000;
			cell.initialized = true;
			return cell;
		}
		
		if (name.contains(".a")) {
			name = name.replace(".a", "[").replace(".b", "]");
		}
		
		int sum = 0;
		name = name.replace("@", "");
		String[] parts =  null;
		
		if (name.contains("[")) {
			parts = name.split("[\\[\\]]");
			name = parts[0];
		}
		
		int i;
		for (i=0; i < space.size() && !space.get(i).name.equals(name); i++) {
			sum += space.get(i).size * space.get(i).size2;
		}
		
		if (parts != null) {
			if (parts.length == 2) {
				sum += Integer.parseInt(parts[1]);
			}
			if (parts.length == 3) {
				sum += Integer.parseInt(parts[2]) + Integer.parseInt(parts[1]);
			}
		}
		
		AddressCell cell = new AddressCell();
		cell.cell = sum;
		cell.initialized = true;
		return cell;
	}


	public void add(String name, int size, int posun) {
		Point x = new Point();
		x.name = name;
		x.posun = posun;
		x.size = size - posun;
		space.add(x);
	}

	public void add(String name, int size1, int size2, int posun, int posun2) {
		Point x = new Point();
		x.name = name;
		x.size = size1 - posun;
		x.size2 = size2 - posun;
		x.posun = posun;
		x.posun2 = posun2;
		space.add(x);
	}

	class Point {
		String name;
		int size;
		int size2 = 1;
		int posun, posun2;
	}
	
	// adapter pre ziskanie hodnoty do stareho kodu
	class AddressCell extends SimpleVariable {
		int cell;
		
		@Override
		public int getValue(int s) {
			return cell;
		}
		
		@Override
		public String getValue(String s) {
			return Integer.toString(cell);
		}
	}
}
