package background.commands;


import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import background.CommandManager;
import background.expressions.LogicalTree;
import background.expressions.MathTree;
import background.expressions.StringProcess;
import background.systemVariables.ArrayVariable;
import background.systemVariables.PointerVariable;
import background.systemVariables.SimpleVariable;
import background.systemVariables.TableVariable;

public class Operation extends Command {	
	public MathTree mathValue;
	public LogicalTree logicValue;
	public StringProcess stringValue;
	public String name;
	public Boolean array = false;
	public Boolean table = false;
	public String constantBool;
	public String s;
	
	public ArrayList<String> rewriteNames;
	
	public boolean pointerChange = false;

	public Operation(Boolean viz, CommandManager cm) { super(viz, cm); }
	public Operation(CommandManager cm) { super(cm); }	
	
	@Override
	public String getType() {
		return "Operation";
	}

	
	public void decomposeValue(String s, String name) {
		this.s = s;
		mathValue = expressionInterpreter.getMath(s);		 
		logicValue = expressionInterpreter.getLogic(s); 
		stringValue = expressionInterpreter.getString(s);
	}
	
	@Override
	public void applyCommand() {
		
		String oldName = null;
		String oldName2 = null;
		
		if (s.contains("alloc")) {
			String regexString = Pattern.quote("alloc(") + "(.*?)" + Pattern.quote(")");
			Pattern pattern = Pattern.compile(regexString);
			Matcher matcher = pattern.matcher(s);

			while (matcher.find()) {
			  Integer bytes = Integer.parseInt(matcher.group(1));
			  int pos = alloc(bytes);
			  
			  s = s.replaceFirst("alloc\\("+bytes+"\\)", Integer.toString(pos));
			}
			decomposeValue(s, null);
		}
		if (s.contains(".s") && !s.contains(" ")) {
			String varName = s.split(".s")[0];
			Boolean addSymbol = false;
			oldName2 = s;
			if (s.contains("$")) {
				addSymbol = true;
				s = s.replace("$", "");
				varName = varName.replace("$", "");
			}
			
			SimpleVariable a = variableManager.getVariableByName(varName);
			if (a != null && (a instanceof PointerVariable)) {
			
				// je to pointer a treba to rewrite
				String var = s.split(".s")[0];
				PointerVariable v = (PointerVariable) variableManager.getVariableByName(var);
			
				if (v.pointer == null) {
					variableManager.errorHandler.errorHappened("Pointer");
					return ;
				} 
				
				s = s.replaceFirst(var, v.pointer.name);
			}
			if (addSymbol) s = "$" + s;
			
			decomposeValue(s, null);
		}
		if (s.contains(".ai.b")) {
			int i = variableManager.getVariableByName("i").getValue(1);
			oldName2 = s;
			String s1 = s.replace(".ai.b", ".a" + i + ".b");
			decomposeValue(s1, null);
		}
		if (pointerChange) {
			PointerVariable pointer = (PointerVariable) variableManager.getVariableByName(name);
			if (pointer == null) variableManager.errorHandler.errorHappened("Initialized");
			else {
				int pos = Integer.parseInt(mathValue.getTreeValue().toString());
				pointer.setPointer(variableManager.space.getVariable(pos));
			}
			if (oldName2 != null) {
				s = oldName2;
				decomposeValue(s, null);
			}
			return;
		}
				
		String type = "";
		if (table) {TableVariable table = variableManager.getTableVariableByName(name); if (table!=null) type = table.getType(); }
		else if (array) {ArrayVariable array = variableManager.getArrayVariableByName(name); if (array!=null) type = array.getType(); }
		else {
			
			if (name.contains(".ai.b")) {
				int i = variableManager.getVariableByName("i").getValue(1);
				oldName = name;
				name = name.replace(".ai.b", ".a" + i + ".b");
			}
			
			if (name.startsWith("*") && name.contains(".s")) {
				// je to pointer a treba to rewrite
				oldName = name;
				String var = name.split(".s")[0].substring(1);
				PointerVariable v = (PointerVariable) variableManager.getVariableByName(var);
				
				if (v.pointer == null) {
					variableManager.errorHandler.errorHappened("Pointer");
					return ;
				} 
				
				name = name.replace("*" + var, v.pointer.name);
				//s = s.replace(name, newName);
				
				//decomposeValue(s, null);
			}
			
			SimpleVariable var = variableManager.getVariableByName(name); 
			if (var!=null) type = var.getType(); 
			}
		if (!type.isEmpty()) {
			String s = "";
			
			switch (type) {
				case "boolean": s = logicValue.getTreeValue().toString(); break; 
				case "int": s = mathValue.getTreeValue().toString(); break;
				case "string": s = stringValue.getStringValue(); break;
			}
			
			if (table) variableManager.updateTableVariable(name, index2.getTreeValue().intValue(), index.getTreeValue().intValue(), s);
			else if (array) variableManager.updateArrayVariable(name, index.getTreeValue().intValue(), s);
			else variableManager.updateVariable(name, s);
		}
		
		if (oldName != null) {
			name = oldName;
		}
		if (oldName2 != null) {
			name = oldName;
		}
	}

	
}
