package background.commands;

import visualization.VisualPanel;
import background.CommandManager;
import background.expressions.MathTree;
import background.systemVariables.ArrayVariable;
import background.systemVariables.PointerVariable;
import background.systemVariables.SimpleVariable;
import background.systemVariables.StructVariable;
import background.systemVariables.TableVariable;

public class VariableCommand extends Command {
	public String type;
	public String name;
	public Boolean array = false;	
	public Boolean table = false;
	public Boolean struct = false;
	public MathTree size;
	public MathTree size2;	
	public VisualPanel visualPanel;
	public int posun = 0;
	public int posun2 = 0;
	public int sizeNumber;
	public int sizeNumber2;
	public boolean pointer = false;
	public Define define = null;
	
	public VariableCommand(Boolean viz, CommandManager cm) { super(viz, cm); visualPanel = commandManager.visualPanel;}
	public VariableCommand(CommandManager cm) { super(cm);}
		
	@Override
	public String getType() {
		return "Variable";
	}
	
	public void setSize(String s) {
		size = expressionInterpreter.getMath(s);	
	}
	public void setSize2(String s) {
		size2 = expressionInterpreter.getMath(s);
	}

	
	@Override
	public void applyCommand() {
		if ((array || table) && (size!=null)) sizeNumber = size.getTreeValue();
		if (table && (size2!=null)) sizeNumber2 = size2.getTreeValue();
		
		if (type == "Integer") type = "int";
		
		if (table) {
			variableManager.addTable(name, posun, sizeNumber, posun2, sizeNumber2, type, vizualize, pointer, define);
			if (vizualize) addVisual(variableManager.getTableVariableByName(name));
		} else if (array) {
			variableManager.addArray(name, posun, sizeNumber, type, vizualize, pointer, define);
			if (vizualize) addVisual(variableManager.getArrayVariableByName(name)); 
		} else if (pointer) {
			 variableManager.addPointer(name, type, null, vizualize);
			 if (vizualize) addVisual(variableManager.getPointerVariableByName(name));
		} else if (struct) {
			StructVariable sv = new StructVariable(variableManager);
			
			for (String var : define.variables) {
				String parts[] = var.trim().split(" ");
				if (!parts[1].contains("*")) {
					variableManager.addVariable(name + ".s" + parts[1], parts[0], null, false);
				} else {
					parts[1] = parts[1].replace("*", "");
					variableManager.addPointer(name + ".s" + parts[1], parts[0], null, false);
				}
				sv.addName(name + ".s" + parts[1]);
			}
			
			sv.setVizualize(true);
			variableManager.addVariable(name, sv);
			if (vizualize) addVisual(sv);	
		} else {
			variableManager.addVariable(name, type, null, vizualize);
			if (vizualize) addVisual(variableManager.getVariableByName(name));						
		}
		
	}
	
	public void addVisual(SimpleVariable variable) { visualPanel.addVisualComponent(variable, null, null, null, params); }
	public void addVisual(TableVariable variable) { visualPanel.addVisualComponent(null, null, variable, null, params); }
	public void addVisual(ArrayVariable array) { visualPanel.addVisualComponent(null, array, null, null, params); }
	public void addVisual(PointerVariable pointer) { visualPanel.addVisualComponent(null, null, null, pointer, params); }
	

	

}
