package language.code;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import language.CodeLanguage;
import background.CommandManager;
import background.VariableManager;
import background.commands.Command;
import background.commands.Cycle;
import background.commands.Define;
import background.commands.IfSegment;
import background.commands.Operation;
import background.commands.Read;
import background.commands.VariableCommand;
import background.commands.Write;

public class CTypeCode extends CodeLanguage {

	public CTypeCode(CommandManager cm) { 
		super(cm); 
		pointerNames = new ArrayList<String>();
	}
	
	public ArrayList<String> pointerNames;
	public ArrayList<String> structNames;
	public HashMap<String, Define> defines;
	public ArrayList<String> rewriteNames;
	
	public void reset() {
		pointerNames = new ArrayList<String>();
		defines = new HashMap<String, Define>();
		structNames = new ArrayList<String>();
		rewriteNames = new ArrayList<String>();
	}
	
	@Override
	public void createDictionary() {
		for (String type: VariableManager.getVariableTypes()) {
			dictionary.put(type, "premenna");
		}				
		addToDictionary("for", "for");
		addToDictionary("if", "if");
		addToDictionary("while", "while");		
		addToDictionary("scanf", "vstup");
		addToDictionary("printf", "vystup");
		addToDictionary("define", "definuj");
	}
	
	@Override
	public String getRegexForUpdate() {
		return ".*[a-zA-Z\\]] *=.*"; 			// je daco, co konci pismenom alebo koncom hranatej zatvorky, za tym moze a nemusi byt medzera, potom rovnitko, potom hocico
	}

	@Override
	public String getStringForUpdate() {
		return "="; 			// je daco, co konci pismenom alebo koncom hranatej zatvorky, za tym moze a nemusi byt medzera, potom rovnitko, potom hocico
	}
	
	@Override
	public String getEOLRegex() {
		return ";";
	}
	
	@Override
	public void recognizeData(String s, Command c) {
		c.vizualize = false;
		
		if (!c.getType().equals("Variable")) s = handleAlloc(handlePointerArrays(handleStructs(s.trim())), c);
		
		switch (c.getType()) {
			case "Variable": {	/* Format: int meno; */			
				VariableCommand c2 = (VariableCommand) c;				
				String parts[] = s.split("[ ]+"); 
				c2.type = parts[0].trim();
				
				if (defines.containsKey(parts[0].trim()) && parts[1].contains("[") && !s.contains("*")) { 				// pole structov
					String subparts[] = parts[1].trim().split("[\\[\\]]");
					c2.name = subparts[0].trim();
					c2.array = true;
					c2.struct = true;
					c2.define = defines.get(parts[0].trim());
					c2.setSize(subparts[1].trim());					
					if (subparts.length>2) {
						c2.array = false;
						c2.table = true;
						c2.setSize2(subparts[3].trim());
					}
					structNames.add(subparts[0].trim());
					

					// ak obsahuje pointer treba ho pridat do pointer names
					for (String var : c2.define.variables ) {
						if (var.contains("*") && subparts.length <= 2) {
							for (int i=0; i<30; i++) {
								String parts2 [] = var.trim().split(" ");
								parts2[1] = parts2[1].replace("*", "");
								pointerNames.add(c2.name + ".a" + i + ".b.s" + parts2[1]);
								rewriteNames.add(c2.name + ".a" + i + ".b.s" + parts2[1]);
							}
						}
						if (var.contains("*") && subparts.length > 2) {
							for (int i=0; i<30; i++) {
								for (int j=0; j<30; j++) {
									String parts2 [] = var.trim().split(" ");
									parts2[1] = parts2[1].replace("*", "");
									pointerNames.add(c2.name + ".a" + i + ".b.a" + j + ".b.s" + parts2[1]);
									rewriteNames.add(c2.name + ".a" + i + ".b.a" + j + ".b.s" + parts2[1]);
								}
							}
						}
					}

			 	} else if (defines.containsKey(parts[0].trim()) && !s.contains("*")) {		// bude to struct
					System.out.println(parts[1]);
					c2.name = parts[1].trim();
					c2.struct = true;
					c2.define = defines.get(parts[0].trim());
					structNames.add(parts[1].trim());
					
					// ak obsahuje pointer treba ho pridat do pointer names
					for (String var : c2.define.variables ) {
						if (var.contains("*")) {
							String parts2 [] = var.trim().split(" ");
							parts2[1] = parts2[1].replace("*", "");
							pointerNames.add(c2.name + ".s" + parts2[1]);
							rewriteNames.add(c2.name + ".s" + parts2[1]);
						}
					}
					
				} else if (parts[1].contains("[") && !parts[1].contains("*")) {		// bude to inicializacia pola
					String subparts[] = parts[1].trim().split("[\\[\\]]");
					c2.name = subparts[0].trim();
					c2.array = true;
					c2.setSize(subparts[1].trim());					
					if (subparts.length>2) {
						c2.array = false;
						c2.table = true;
						c2.setSize2(subparts[3].trim());
					}
				} else if(parts[1].contains("[") && parts[1].contains("*")) {	// pole smernikov
					String subparts[] = parts[1].trim().split("[\\[\\]]");
					c2.name = subparts[0].trim().substring(1);
					c2.array = true;
					c2.pointer = true;
					c2.setSize(subparts[1].trim());					
					if (subparts.length>2) {
						c2.array = false;
						c2.table = true;
						c2.setSize2(subparts[3].trim());
					}
					pointerNames.add(subparts[0].trim().substring(1));
					
				} else if (parts[1].contains("*")) {				// bude to inicializacia smernika
					
					c2.pointer = true;
					c2.name = parts[1].substring(1);
					this.pointerNames.add(parts[1].substring(1));
					
				} else {											// bude to inicializacia zakladnej premennej
					c2.name = parts[1].trim();				
				}
				break;
			}
			case "Define": {
				String[] parts = s.split(":");
				
				String name = parts[0].replaceAll("define *", "");
				String[] variables = parts[1].split(",");
				
				for (int i=0; i<variables.length; i++) {
					if (variables[i].contains("*") && variables[i].trim().startsWith(name.trim())) {
						variables[i] = variables[i].replaceFirst(name.trim(), "struct");
					}
				}
				
				((Define)c).setParameters(name, variables);
				
				defines.put(name.trim(), (Define)c);
				dictionary.put(name.trim(), "premenna");
				
				break;
			}
			case "Operation": {	/* Format: i=i+10; */			
				Operation c2 = (Operation) c;	
				c2.rewriteNames = rewriteNames;
				
				String parts[] = s.split(this.getStringForUpdate(), 2); 
				
				// pripravime pointrove operacie
				parts[1] = processPointers(parts[1]);
				
				for (String x : pointerNames) {
					if (parts[0].trim().charAt(0) != '*' && (parts[0].trim().equals(x) || 
							(parts[0].trim().startsWith(x+".a")))) parts[0] = '@' + parts[0];
						
				}

				c2.s = parts[1].trim();
				
				if (parts[0].contains("[")) {						// bude to uprava pola
					String subparts[] = parts[0].trim().split("[\\[\\]]");
					c2.name = subparts[0].trim();				
					c2.array = true;					
					c2.setIndex(subparts[1].trim());
					if (subparts.length>2) {
						c2.array = false;
						c2.table = true;
						c2.setIndex2(subparts[3].trim());
					}

				} else if(parts[0].charAt(0) == '@') { 				// priradenie hodnoty do smernika
					
					c2.name = parts[0].substring(1).trim();
					c2.pointerChange = true;
					//parts[1] = parts[1] + " + 0";
					
				} else {			 						// bude uprava zakladnej premennej
					c2.name = parts[0].trim();				
				}

				c2.decomposeValue(parts[1].trim(), c2.name);
				break;
			}
			
			case "If": {	/* Format: if (podmienka==true) */			
				IfSegment c2 = (IfSegment) c;				
				String parts[] = s.split(" ", 2); 	
				parts[1] = processPointers(parts[1]);
				String x = parts[1].substring(1, parts[1].length()-1);		// odstranim vonkajsie zatvorky				
				c2.setCondition(x);
				break;
			}
			case "While": {	/* Format: while (podmienka==true) */			
				Cycle c2 = (Cycle) c;				
				String parts[] = s.split(" ", 2); 		
				parts[1] = processPointers(parts[1]);		
				String x = parts[1].substring(1, parts[1].length()-1);		// odstranim vonkajsie zatvorky
				c2.setCondition(x);
				break;
			}
			case "For": {	/* Format: for (i=1; i<=5; i=i+1) */			
				Cycle c2 = (Cycle) c;	
				String parts[] = s.split(" ", 2);
				
				String subparts[] = parts[1].substring(1, parts[1].length()-1).split(",");
				
				subparts[1] = processPointers(subparts[1]);

				Operation init = (Operation) commandManager.getCommand("Operation", c2.vizualize);
				recognizeData(subparts[0].trim(), init);
				c2.init = init;
				init.codeLine = c2.codeLine;
				
				c2.setCondition(subparts[1].trim());

				Operation inc = (Operation) commandManager.getCommand("Operation", c2.vizualize);
				recognizeData(subparts[2].trim(), inc);
				c2.increment = inc;
				inc.codeLine = c2.codeLine;
				break;
			}
			case "Read": {   /* Format: scanf(%d, number) */
				Read c2 = (Read) c;
				String parts[] = s.split("[,)]");

				parts[1] = processPointers(parts[1]);
				
				if (parts[1].contains("[")) {						// bude to pole
					String subparts[] = parts[1].trim().split("[\\[\\]]");
					c2.name = subparts[0].trim();				
					c2.array = true;					
					c2.setIndex(subparts[1].trim());
					if (subparts.length>2) {
						c2.array = false;
						c2.table = true;
						c2.setIndex2(subparts[3].trim());
					}
				} else {			 						// bude zakladna premenna
					c2.name = parts[1].trim();				
				}								
				break;
				
			}
			case "Write": {   /* Format: printf("string")   OR   printf(variable) */
				Write c2 = (Write) c;
				String parts[] = s.split("[()]");
				parts[1] = processPointers(parts[1]);
				parts[1] = parts[1].trim();

				if (parts[1].contains("[")) {						// bude to pole
					String subparts[] = parts[1].trim().split("[\\[\\]]");
					c2.name = subparts[0].trim();
					c2.array = true;					
					c2.setIndex(subparts[1].trim());
					if (subparts.length>2) {
						c2.array = false;
						c2.table = true;
						c2.setIndex2(subparts[3].trim());
					}
				} else if (parts[1].startsWith("\"") && parts[1].endsWith("\"")) {		 						
					c2.value = parts[1].replace("\"", "");;
				} else {											// bude zakladna premenna
					c2.name = parts[1].trim();				
				}								
				break;
				
			}
		}
	}
	
	protected String handleStructs(String s) {
		s = s.replace("->", ".s");
		String parts[] = s.split(" |\\-|\\+|\\*|%|<|>|=|,|\\(");
		
		for (String name : structNames) {
			for (String part : parts) {
				if (part.contains(name)) {
					String fix = part.replace("[", ".a").replace("]", ".b");
					
					s = s.replace(part, fix);
				}
			}
		}
		
		
		return s;
	}

	protected String handlePointerArrays(String s) {
		// uprava poli pointrov, trochu sialene napisane
		String parts[] = s.split(" |\\-|\\+|\\*|%|<|>|=|,|\\(");
		for (String name : pointerNames) {
			
			
			for(String part : parts) {
				if (part.contains(name)) {
					String fix = part.replace("[", ".a").replace("]", ".b");
					
					s = s.replace(part, fix);
				}
			}
		}
		return s;
	}
	
	protected String handleAlloc(String s, Command c) {
		return s;
	}

	protected String processPointers(String s)  {
		
		// n = 4; sa prelozi na $n=4;
		// &n sa prelozi na @n
		// *n sa prelozi na n
		
		for (String name : pointerNames) {		// $pointer znaci jeho adresu kam ukazuje. teda jeho hodnotu keby bol int
			s = s.replaceAll("(?<![a-zA-Z0-9\\*\\&])" + name + "(?![a-zA-Z0-9])", "\\$" + name);
		}
		
		for (String name : pointerNames) {		// pointer znaci hodnoty na adrese kam ukazuje
			s = s.replaceAll("(?<![a-zA-Z0-9]) *\\*" + name, name);				// ak chceme hodnotu
		}
		
		s = s.replaceAll("&(?!&)", "@");		// @pointer znaci ziskanie adresy pointra v adresnom priestore
		
		
		return s;
	}
	
	@Override
	public String changeFormat(String s) {
		return formatProcess(s, "{", "}");
	}	
}
