package language.code;

import java.util.ArrayList;
import java.util.HashMap;

import language.CodeLanguage;
import background.CommandManager;
import background.VariableManager;
import background.commands.Command;
import background.commands.Cycle;
import background.commands.Define;
import background.commands.IfSegment;
import background.commands.Operation;
import background.commands.Read;
import background.commands.VariableCommand;
import background.commands.Write;

public class Pascal extends CodeLanguage {

	public Pascal(CommandManager cm) { super(cm); }
	
	@Override
	public void createDictionary() {
		for (String type: VariableManager.getVariableTypes()) {
			dictionary.put(type, "premenna");
		}
		addToDictionary("var", "premenna");
		addToDictionary("for", "for");
		addToDictionary("if", "if");
		addToDictionary("while", "while");
		addToDictionary("repeat", "repeat");		
		addToDictionary("read", "vstup");
		addToDictionary("write", "vystup");
		addToDictionary("readln", "vstup");
		addToDictionary("writeln", "vystup");
		addToDictionary("TYPE", "definuj");

	}
	
	@Override
	public String getRegexForUpdate() {
		return ".*[a-zA-Z\\]]\\^? *:=.*"; 			// je daco, co konci pismenom alebo koncom hranatej zatvorky, za tym moze a nemusi byt medzera, potom rovnitko, potom hocico
	}

	@Override
	public String getStringForUpdate() {
		return ":="; 			// je daco, co konci pismenom alebo koncom hranatej zatvorky, za tym moze a nemusi byt medzera, potom rovnitko, potom hocico
	}
	
	@Override
	public String getEOLRegex() {
		return ";";
	}
	
	ArrayList<String> pointerNames = new ArrayList<String>();
	private HashMap<String, Define> defines;
	private ArrayList<String> structNames;
	
	@Override
	public void reset() {
		pointerNames = new ArrayList<String>();
		defines = new HashMap<String, Define>();
		structNames = new ArrayList<String>();
	}
	
	@Override
	public void recognizeData(String s, Command c) {
		c.vizualize = false;
		s = s.trim();
		
		if (!c.getType().equals("Variable")) s = (handlePointerArrays(handleStructs(s)));
		
		switch (c.getType()) {
		
			case "Variable": {		/* Format: var meno : int; */			
				VariableCommand c2 = (VariableCommand) c;				
				String parts[] = s.split("[ :]+"); 
				c2.name = parts[1].trim();
				
				if (defines.containsKey(parts[parts.length-1].trim()) && parts[2].toLowerCase().contains("array")) {
					
					String subparts[] = parts[2].replace("integer", "int").trim().split("[\\[\\] \\.,]+");
					c2.posun = Integer.parseInt(subparts[1].trim());
					c2.sizeNumber = Integer.parseInt(subparts[2].trim())+1;
					c2.define = defines.get(parts[4].trim());
					if (subparts.length>3) {
						c2.table = true;
						c2.posun2 = Integer.parseInt(subparts[3].trim());
						c2.sizeNumber2 = Integer.parseInt(subparts[4].trim())+1;						
					} else { 
						c2.array = true;						
					}	
					
					if (parts[4].contains("^")) {
						c2.pointer = true;
						pointerNames.add(parts[1].trim());
					}
					c2.type = parts[4].trim().replace("^", "").toLowerCase();	
					structNames.add(parts[1].trim());
					

					// ak obsahuje pointer treba ho pridat do pointer names
					for (String var : c2.define.variables ) {
						if (var.contains("*") && subparts.length <= 3) {
							for (int i=0; i<30; i++) {
								String parts2 [] = var.trim().split(" ");
								parts2[1] = parts2[1].replace("*", "");
								pointerNames.add(c2.name + ".a" + i + ".b.s" + parts2[1]);
							}
						}
						if (var.contains("*") && subparts.length > 3) {
							for (int i=0; i<30; i++) {
								for (int j=0; j<30; j++) {
									String parts2 [] = var.trim().split(" ");
									parts2[1] = parts2[1].replace("*", "");
									pointerNames.add(c2.name + ".a" + i + ".b.a" + j + ".b.s" + parts2[1]);
								}
							}
						}
					}
					
				} else if (defines.containsKey(parts[2].trim())) {		// bude to struct
					System.out.println(parts[1]);
					c2.name = parts[1].trim();
					c2.struct = true;
					c2.define = defines.get(parts[2].trim());
					c2.type = parts[2].trim();
					
					for (String x : c2.define.variables) {
						if (x.contains("*")) {
							String parts2[] = x.split(" ");
							pointerNames.add(c2.name + ".s" + parts2[1].replace("*", ""));
						}
					}
				} else if (parts[2].toLowerCase().contains("array")) {
					String subparts[] = parts[2].trim().split("[\\[\\] \\.,]+");
					c2.posun = Integer.parseInt(subparts[1].trim());
					c2.sizeNumber = Integer.parseInt(subparts[2].trim())+1;
					if (subparts.length>3) {
						c2.table = true;
						c2.posun2 = Integer.parseInt(subparts[3].trim());
						c2.sizeNumber2 = Integer.parseInt(subparts[4].trim())+1;						
					} else { 
						c2.array = true;						
					}	
					
					if (parts[4].contains("^")) {
						c2.pointer = true;
						pointerNames.add(parts[1].trim());
					}
					c2.type = parts[4].trim().replace("^", "").toLowerCase();			
					
				} else if(s.contains("^")) { 				// bude inicializacia pointra
					c2.pointer = true;
					c2.name = parts[1].trim();
					c2.type = parts[2].substring(1).trim();
					this.pointerNames.add(parts[1].trim());
					
				} else {									// bude inicializacia zakladnej premennej
					c2.type = parts[2].trim();				
				}
				
				if (c2.type != null && c2.type.toLowerCase().equals("integer")) c2.type="int";
				
				break;
			}			
			case "Define": {
				String[] parts = s.split("=");
				Define c2 = (Define) c;
				String name = parts[0].replaceAll("TYPE *", "");
				String[] variables = parts[1].replace("integer", "int").replace(":", " ").replace("^", "*").split(",");
				String [] var2 = new String[variables.length];
				
				for (int i=0; i<variables.length; i++) {
					String p[] = variables[i].trim().split(" ", 2);
					
					if (p[1].contains("*")) {
						p[0] = "*" + p[0];
						p[1] = p[1].replace("*", "");
					}
					
					var2[i] = p[1].trim() + " " + p[0].trim();
				}
				
				c2.setParameters(name.trim(), var2);
				
				defines.put(name.trim(), c2);
				dictionary.put(name.trim(), "premenna");
				
				break;
			}
			case "Operation": {	/* Format: i=i+10; */			
				Operation c2 = (Operation) c;		
								
				String parts[] = s.split(this.getStringForUpdate(), 2);

				parts[1] = processPointers(parts[1]);

				for (String x : pointerNames) {
					if (parts[0].trim().charAt(parts[0].trim().length()-1) != '^' && (parts[0].trim().equals(x) ||
							parts[0].trim().startsWith(x + ".a"))) parts[0] = '@' + parts[0];
				}
				
				if (parts[0].contains("[")) {						// bude to uprava pola
					parts[0] = parts[0].replace(",", "][");
					String subparts[] = parts[0].trim().split("[\\[\\]]");
					c2.name = subparts[0].trim();
					c2.array = true;					
					c2.setIndex(subparts[1].trim());
					if (subparts.length>2) {
						c2.array = false;
						c2.table = true;
						c2.setIndex2(subparts[3].trim());
					}

				} else if(parts[0].charAt(0) == '@') { 				// priradenie hodnoty do smernika
					
					c2.name = parts[0].substring(1).trim();
					c2.pointerChange = true;
					
				} else {			 						// bude uprava zakladnej premennej
					c2.name = parts[0].trim();				
				}

				c2.decomposeValue(parts[1].trim(), c2.name);
				break;
			}
			case "If": {	/* Format: if podmienka==true then */			
				IfSegment c2 = (IfSegment) c;				
				String parts[] = s.split("[ ]+", 2);
				parts[1] += " ";
				parts[1] = parts[1].replace(" then ", "");
				parts[1] = processPointers(parts[1]);
				String x = parts[1].trim();		// odstranim vonkajsie zatvorky				
				c2.setCondition(x);
				break;
			}
			case "While": {	/* Format: while podmienka==true do*/			
				Cycle c2 = (Cycle) c;				
				String parts[] = s.split("[ ]+", 2);
				parts[1] += " ";
				parts[1] = parts[1].replace(" do ", "");
				parts[1] = processPointers(parts[1]);
				String x = parts[1].trim();		// odstranim vonkajsie zatvorky
				c2.setCondition(x);
				break;
			}
			case "Repeat": {	/* Format: repeat until (podmienka==true) */			
				Cycle c2 = (Cycle) c;				
				String parts[] = s.split("[ ]+", 3); 				
				String x = parts[2].trim();		// odstranim vonkajsie zatvorky
				c2.setCondition(x);
				break;
			}
			case "For": {	/* Format: for i:=1 to 5 do */			
				Cycle c2 = (Cycle) c;	
				s = s.replaceFirst(" *:= *", ":=").replaceFirst("for ", "").trim();			// odstranila som medzery okolo :=, odstranila som for na zaciatku
				s = s+" ";
				s = s.replace(" do ", "").trim();											// odstranila som do na konci
				
				boolean downto = false;
				String parts[];
				if (s.contains(" downto ")) {
					downto = true;
					parts = s.split(" downto ", 2);  // vznikne spodla a vrchna hranica
				} else {
					parts = s.split(" to ", 2);  // vznikne spodna a vrchna hranica
				}
								
				String subparts[] = parts[0].split(this.getStringForUpdate());				// vznikne premenna a spodna hranica				 								
				String variable = subparts[0].trim();
				String first = subparts[1].trim();				
				String second = parts[1].trim();
				
				if (first.contains("+") || first.contains("-") || first.contains("*") || first.contains("/") || first.contains("%")) {
					first = "("+first+")";
				}
								
				Operation init = (Operation) commandManager.getCommand("Operation", c2.vizualize);								
				Operation inc = (Operation) commandManager.getCommand("Operation", c2.vizualize);
				
				//c2.incrementFirst=true;				
								
				if (!downto) {				
					recognizeData(variable+this.getStringForUpdate()+first, init);
					c2.setCondition(variable+"<="+second);
					recognizeData(variable+this.getStringForUpdate()+variable+"+1", inc);					
				} else {
					recognizeData(variable+this.getStringForUpdate()+first, init);
					c2.setCondition(variable+">="+second);
					recognizeData(variable+this.getStringForUpdate()+variable+"-1", inc);					
				}
				
				c2.init = init;
				c2.increment = inc;
				break;
			}
			case "Read": {   /* Format: read(number) */
				Read c2 = (Read) c;
				String parts[] = s.split("[()]");		

				parts[1] = processPointers(parts[1]);
				
				if (parts[1].contains("[")) {						// bude to pole
					parts[1] = parts[1].replace(",", "][");
					String subparts[] = parts[1].trim().split("[\\[\\]]");
					c2.name = subparts[0].trim();				
					c2.array = true;					
					c2.setIndex(subparts[1].trim());
					if (subparts.length>2) {				
						c2.table = true;
						c2.array = false;		
						c2.setIndex2(subparts[3].trim());
					}
				} else {			 						// bude zakladna premenna
					c2.name = parts[1].trim();				
				}								
				break;
				
			}
			case "Write": {   /* Format: write("string")   OR   writeln(variable) */				
				Write c2 = (Write) c;
				String parts[] = s.split("[()]");
				parts[1] = processPointers(parts[1]);
				parts[1] = parts[1].trim();

				
				if (parts[1].contains("[")) {						// bude to pole
					parts[1] = parts[1].replace(",", "][");
					String subparts[] = parts[1].trim().split("[\\[\\]]");
					c2.name = subparts[0].trim();				
					c2.array = true;					
					c2.setIndex(subparts[1].trim());
					if (subparts.length>2) {
						c2.table = true;
						c2.array = false;		
						c2.setIndex2(subparts[3].trim());
					}
				} else if ((parts[1].startsWith("\"") && parts[1].endsWith("\"")) || (parts[1].startsWith("'") && parts[1].endsWith("'"))) {			 						
					c2.value = parts[1].replace("\"", "").replace("''", "\"").replace("'", "").replace("\"", "'");
				} else {											// bude zakladna premenna
					c2.name = parts[1].trim();				
				}								
				
				if (parts[0].trim().equalsIgnoreCase("writeln")) { c2.value = c2.value+System.getProperty("line.separator"); }
				break;
				
			}
		}
	}	
	

		protected String handleStructs(String s) {
			s = s.replace("->", ".s");
			String parts[] = s.split(" |\\-|\\+|\\*|%|<|>|=|,|\\(");

			for (String name : structNames) {
				for (String part : parts) {
					if (part.contains(name)) {
						String fix = part.replace("[", ".a").replace("]", ".b");
						
						s = s.replace(part, fix);
					}
				}
			}

			
			return s;
		}

	protected String handlePointerArrays(String s) {
		// uprava poli pointrov, trochu sialene napisane
		String parts[] = s.split(" |\\-|\\+|\\*|%|<|>|=|,|\\(");
		for (String name : pointerNames) {
			
			
			for(String part : parts) {
				if (part.contains(name)) {
					String fix = part.replace("[", ".a").replace("]", ".b");
					
					s = s.replace(part, fix);
				}
			}
		}
		return s;
	}
	
	private String processPointers(String s) {
		
		for (String name : pointerNames) {							// ak chceme adresu kam ukazuje pointer
			s = s.replaceAll("(?<![a-zA-Z0-9@])" + name + "\\^", "\\$" + name);
		}
		

		// odstran ^ na konci mien v poliach
		String parts[] = s.split(" |\\-|\\+|\\*|%|<|>|=|,|\\(");
		for (String name : pointerNames) {
			for(String part : parts) {
				if (part.startsWith(name) && part.contains(".a") && part.contains("^")) {
					String fix = part.replace("^", "");
					
					s = s.replace(part, fix);
				}
			}
		}
		
		// odstran ^ na konci mien v premennych
		for (String name : pointerNames) {
			s = s.replaceAll("(?<![a-zA-Z0-9]) *" + name + "\\^", name);				
		}
		
		// get adress je v packale @ tak ako my pouzivame
		return s;
	}

	@Override
	public String changeFormat(String s) {
		s = s.replaceFirst("begin", "").replaceFirst("end\\.", "");		
		return formatProcess(s, "begin", "end;");
	}

}
